# -*- coding: utf-8 -*-

# Coded by Takashi Ida, August 23, 2021

import numpy as np
from scipy import special as sp
from scipy import optimize as opt

########################################
# Symmetric Rosin-Rammler type function
########################################
#####################################
# Shape parmeter (h) to kurtosis (k)
#####################################
def fSRR_h2k(h):
	x2 = sp.gamma(2/h+1)
	x4 = sp.gamma(4/h+1)
	kurtosis = x4 / x2**2 - 3
	return kurtosis
######################################
# kurtosis (k) to shape parameter (a)
#  k0: kurtosis (NumPy array)
######################################
def fSRR_k2h(kurtosis):
	h0 = 1.0 # Left side
	h2 = 0.72 # Right side
	k0 = 3.0 - kurtosis
	k2 = 12.0 - kurtosis
	xTol = 1.0E-8 # Toleance
	nIter = np.ceil(np.log2((h0-h2)/xTol)).astype(int)
	for iIter in range(nIter):
		h1 = 0.5 * (h0 + h2)
		k1 = fSRR_h2k(h1) - kurtosis
		if (np.sign(k0) == np.sign(k1)):
			h0 = h1
			k0 = k1
		else:
			h2 = h1
			k2 = k1
	return h1

################
# Demonstration
################
startPoint = 3.0
endPoint = 13.0
step = 1.0
kList = np.arange(startPoint, endPoint, step)
hList = np.array([])
for i in range(kList.size):
	k = kList[i]
	h = fSRR_k2h(k)
	hList = np.append(hList,h)
print('k = ', kList)
print('h = ', hList)
